function Feats = m_cmpInitFeats_1D(metaD, posOrNeg)
% Compute initial features
% Inputs:
%   metaD: a structure with the required fields:
%       PosD: a cell structure containing positive signals
%       NegD: a cell structure containing negative signals
%       initOption_pos, initOption_neg: options for positive or negative classes. 
%       nBin: number of histogram bin = # of dimension of the SVM weight vector.
%   posOrNeg: must be either 'pos' or 'neg'
% Outputs:
%   Feats: nBin*n matrix, where n is the number of positive or negative signals
%       depending on posOrNeg option.
% By: Minh Hoai Nguyen (minhhoai@gmail.com)
% Last modified: 3 Oct 2010

nBin = metaD.nBin;

if strcmpi(posOrNeg, 'pos') && strcmpi(metaD.initOption_pos, 'initW')
    initW = metaD.initW;
    Feats = m_cmpSegFeats_1D(metaD, initW, 'pos');
    return;
end

if strcmpi(posOrNeg, 'pos')
    Signals = metaD.PosD; 
    option = metaD.initOption_pos;
elseif strcmp(posOrNeg, 'neg')
    Signals = metaD.NegD;
    option = metaD.initOption_neg;
else
    error('m_cmpSegFeats_1D: unrecognized option');
end;

Feats = zeros(nBin, length(Signals));
if strcmpi(option, 'global')
    for i=1:length(Signals)
        signalL = max(Signals{i}(:,1)); % length of signal
        histo = hist(Signals{i}(:,2), 1:nBin);
        Feats(:,i) = histo(:)/signalL;
    end;
elseif strcmpi(option, 'random')
    rand('twister', sum(100*clock));
    for i=1:length(Signals)
        signal = Signals{i};
        signalL = max(signal(:,1));
        t1t2 = ml_rand(1,2,1,signalL, 1);
        t1 = min(t1t2);
        t2 = max(t1t2);
        idxs = and(signal(:,1) >= t1, signal(:,2) <= t2);
        histo = hist(signal(idxs,2), 1:nBin);
        Feats(:,i) = histo(:)/signalL;
    end;
end;



