function Feats = m_cmpInitFeats_2D(metaD, posOrNeg)
% Comptue the initial feature vectors.
% Inputs:
%   metaD is structure with require fields:
%       featFiles_pos, featFiles_neg: pointers to feature files
%       initOption_pos, initOption_neg: options for initial feature vectors of 
%           positive and negative classes.
%       szRng_pos, szRng_neg: range for postive and negative windows
%       nBranch, nLevel: options for feature computation
%       nBin: # of dimension = length of the weight vector.
%   posOrNeg: valid options are 'pos' or 'neg'. 
% Outputs:
%   Feats: nBin*n matrix, where n is the number of positive or negative signals
%       depending on posOrNeg option.
% By Minh Hoai Nguyen (minhhoai@gmail.com)
% Last modified: 3 Oct 2010

if strcmpi(posOrNeg, 'pos')
    imFiles = metaD.featFiles_pos;
    option = metaD.initOption_pos;
    if strcmpi(option, 'initW')
        Feats = m_cmpSegFeats_2D(metaD, metaD.initW, 'pos'); 
        return;
    end
elseif strcmp(posOrNeg, 'neg')
    imFiles = metaD.featFiles_neg;
    option = metaD.initOption_neg;
else
    error('m_cmpInitFeats_2D: unknown option for posOrNeg');
end;

nBin = metaD.nBin;
nBranch = metaD.nBranch;
nLevel = metaD.nLevel;
    

isKpAssign = 1;
nIm = length(imFiles);
k = (nBranch^nLevel - 1)/(nBranch - 1);
Feats = zeros(nBin*k, nIm);


if strcmp(option, 'GT') % Starting with Ground truth
    for i=1:nIm
        im = m_readFeatFile_2D(imFiles{i});
        gt = imread(gtFiles{i});
        gt = gt(:,:,1);
        gt = imresize(gt, size(im));
        gt = gt(:,:,1) == 255;
         
        BBox = regionprops(double(gt), 'BoundingBox');
        BBox = BBox.BoundingBox;
        xStart = ceil(BBox(1));
        yStart = ceil(BBox(2));
        xEnd = floor(BBox(1) + BBox(3));
        yEnd = floor(BBox(2) + BBox(4));

        leafHists = ml_cmpLeafHists(im, nBin, [xStart yStart xEnd yEnd],...
            nBranch, nLevel, isKpAssign);
        histPyr = ml_leafHists2histPyr(leafHists, nBranch, nLevel);
        Feats(:,i) = histPyr(:);

%         Feats(:,i) = Feats(:,i)./sum(Feats(:,i));
    end;
elseif strcmpi(option, 'GT-CMUFace') || strcmpi(option, 'GT-PittsburghCar')
    % Starting with Ground truth
    for i=1:nIm
        im = m_readFeatFile_2D(imFiles{i});        
        [path,name] = fileparts(imFiles{i}) ;        
        imIdx = str2double(name(1:4));
        if isnan(imIdx)
            imIdx = str2double(name(7:10));
        end;
        load(sprintf('%s/%04d.mat', gtFiles, imIdx), 'coordMin', 'coordMax');
        xStart = coordMin(1);
        yStart = coordMin(2);
        xEnd = coordMax(1) ;
        yEnd = coordMax(2) ;
        
        leafHists = ml_cmpLeafHists(im, nBin, [xStart yStart xEnd yEnd],...
            nBranch, nLevel, isKpAssign);
        histPyr = ml_leafHists2histPyr(leafHists, nBranch, nLevel);
        Feats(:,i) = histPyr(:);
    end;    
elseif strcmpi(option, 'GT-CMUFace++') || strcmpi(option, 'GT-PittsburghCar++')
    % Starting with Ground truth
    for i=1:nIm
        im = m_readFeatFile_2D(imFiles{i});        
        [path,name] = fileparts(imFiles{i}) ;        
        imIdx = str2double(name(1:4));
        if isnan(imIdx)
            imIdx = str2double(name(7:10));
        end;
        load(sprintf('%s/%04d.mat', gtFiles, imIdx), 'coordMin', 'coordMax');
        xStart = coordMin(1);
        yStart = coordMin(2);
        xEnd = coordMax(1) ;
        yEnd = coordMax(2) ;
        
        boxW = xEnd - xStart + 1;
        boxH = yEnd - yStart + 1;
        wExt = ceil(boxW/2);
        hExt = ceil(boxH/2);
        xStart = xStart - wExt;
        xEnd = xEnd + wExt;
        yStart = yStart - hExt;
        yEnd = yEnd + hExt;
        xStart = max(xStart, 1);
        yStart = max(yStart, 1);
        
        leafHists = ml_cmpLeafHists(im, nBin, [xStart yStart xEnd yEnd],...
            nBranch, nLevel, isKpAssign);
        histPyr = ml_leafHists2histPyr(leafHists, nBranch, nLevel);
        Feats(:,i) = histPyr(:);
    end;    
elseif strcmpi(option, 'GT-Caltech-4') % Starting with Ground truth
    load(gtFiles, 'SubDir_Data');
    
    %loadedGtFile = [];
    for i=1:nIm
        im = m_readFeatFile_2D(imFiles{i});        
        [path,name] = fileparts(imFiles{i}) ;        
        imIdx = str2double(name(1:4));
        if isnan(imIdx)
            imIdx = str2double(name(7:10));
        end;
        
        gt = SubDir_Data(:, imIdx);        
        
        xStart = floor(min(gt(1:2:end)));
        yStart = floor(min(gt(2:2:end)));
        xEnd = ceil(max(gt(1:2:end)));
        yEnd = ceil(max(gt(2:2:end)));

        leafHists = ml_cmpLeafHists(im, nBin, [xStart yStart xEnd yEnd],...
            nBranch, nLevel, isKpAssign);
        histPyr = ml_leafHists2histPyr(leafHists, nBranch, nLevel);
        Feats(:,i) = histPyr(:);

%         Feats(:,i) = Feats(:,i)./sum(Feats(:,i));
    end;    
elseif strcmpi(option, 'GT-Caltech-4++') % Starting with Ground truth
    load(gtFiles, 'SubDir_Data');
    
    %loadedGtFile = [];
    for i=1:nIm
        im = m_readFeatFile_2D(imFiles{i});        
        [path,name] = fileparts(imFiles{i}) ;        
        imIdx = str2double(name(1:4));
        if isnan(imIdx)
            imIdx = str2double(name(7:10));
        end;
        
        gt = SubDir_Data(:, imIdx);        
        
        xStart = floor(min(gt(1:2:end)));
        yStart = floor(min(gt(2:2:end)));
        xEnd = ceil(max(gt(1:2:end)));
        yEnd = ceil(max(gt(2:2:end)));
        boxW = xEnd - xStart + 1;
        boxH = yEnd - yStart + 1;
        wExt = ceil(boxW/2);
        hExt = ceil(boxH/2);
        xStart = xStart - wExt;
        xEnd = xEnd + wExt;
        yStart = yStart - hExt;
        yEnd = yEnd + hExt;
        xStart = max(xStart, 1);
        yStart = max(yStart, 1);

        leafHists = ml_cmpLeafHists(im, nBin, [xStart yStart xEnd yEnd],...
            nBranch, nLevel, isKpAssign);
        histPyr = ml_leafHists2histPyr(leafHists, nBranch, nLevel);
        Feats(:,i) = histPyr(:);

%         Feats(:,i) = Feats(:,i)./sum(Feats(:,i));
    end;       
elseif strcmpi(option, 'Global') % Starting with Global histogram
    for i=1:nIm
        im = m_readFeatFile_2D(imFiles{i});

        if isKpAssign
            imH = ceil(max(im(:,1)));
            imW = ceil(max(im(:,2)));
        end;
        
        leafHists = ml_cmpLeafHists(im, nBin, [1 1 imW imH],...
            nBranch, nLevel, isKpAssign);
        histPyr = ml_leafHists2histPyr(leafHists, nBranch, nLevel);
        Feats(:,i) = histPyr(:);
%         Feats(:,i) = Feats(:,i)./sum(Feats(:,i));
    end;    
elseif strcmpi(option, 'random')
    for i=1:nIm
        im = m_readFeatFile_2D(imFiles{i});

        if isKpAssign
            imH = ceil(max(im(:,1)));
            imW = ceil(max(im(:,2)));
        end;
                        
        xs = ml_rand(1, 2, 1, imW, 0);
        ys = ml_rand(1, 2, 1, imH, 1);
        
        xStart = min(xs);        
        xEnd = max(xs);
        
        yStart = min(ys);        
        yEnd = max(ys);
        %fprintf('   random, im %d, x: [%d %d], y: [%d %d]\n', i, xStart, xEnd, yStart, yEnd);

        leafHists = ml_cmpLeafHists(im, nBin, [xStart yStart xEnd yEnd],...
            nBranch, nLevel, isKpAssign);
        
        
        histPyr = ml_leafHists2histPyr(leafHists, nBranch, nLevel);
        Feats(:,i) = histPyr(:);
        %Feats(:,i) = Feats(:,i)./sum(Feats(:,i));
    end;
elseif strcmpi(option, 'scramble_random') % permute the kp location, random box
    for i=1:nIm
        im = m_readFeatFile_2D(imFiles{i});
        ranNums = randperm(size(im, 1));
        im(:,[1,2]) = im(ranNums, [1,2]); % permute the location of kp

        if isKpAssign
            imH = ceil(max(im(:,1)));
            imW = ceil(max(im(:,2)));
        end;
                        
        xs = ml_rand(1, 2, 1, imW, 0);
        ys = ml_rand(1, 2, 1, imH, 1);
        
        xStart = min(xs);        
        xEnd = max(xs);
        
        yStart = min(ys);        
        yEnd = max(ys);
        %fprintf('   scramble_random, im %d, x: [%d %d], y: [%d %d]\n', i, xStart, xEnd, yStart, yEnd);

        leafHists = ml_cmpLeafHists(im, nBin, [xStart yStart xEnd yEnd],...
            nBranch, nLevel, isKpAssign);
        
        
        histPyr = ml_leafHists2histPyr(leafHists, nBranch, nLevel);
        Feats(:,i) = histPyr(:);
        %Feats(:,i) = Feats(:,i)./sum(Feats(:,i));
    end;
else 
    error('m_cmpInitFeats: unknow option');
end;