function m_main_cmuFace()
% Run experiment on CMU Face dataset

% baseDir = '/cv/lab_space/minhhoan/extra/CMUFaces_all/m_rand-sift/';
% baseDir_im = '/cv/lab_space/minhhoan/CMUFaces_all/';
baseDir = '../data_CMUFaces/CMUFaces_all_sift/';
baseDir_im = '../data_CMUFaces/CMUFaces_all/';


global CLUST_LEVEL;
CLUST_LEVEL = 3;

posClasses = {'glasses'};
negClasses = {'noglasses'};

trIdxs1  = 1:126;
tstIdxs1 = 127:311;
trIdxs2  = 1:128;
tstIdxs2 = 129:313;

posTrIdxss =  {trIdxs1};
negTrIdxss =  {trIdxs2};
posTstIdxss = {tstIdxs1};
negTstIdxss = {tstIdxs2};

[posTrFiles, posTstFiles] = getTrTstLists(baseDir, posClasses, ...
    posTrIdxss, posTstIdxss, 'mat');

[negTrFiles, negTstFiles] = getTrTstLists(baseDir, negClasses, ...
    negTrIdxss, negTstIdxss, 'mat');

[posTrFiles_im, posTstFiles_im] = getTrTstLists(baseDir_im, posClasses, ...
    posTrIdxss, posTstIdxss, 'jpg');

[negTrFiles_im, negTstFiles_im] = getTrTstLists(baseDir_im, negClasses, ...
    negTrIdxss, negTstIdxss, 'jpg');


[dc, nBin] = m_readFeatFile_2D([], 'nHistBinOnly');
metaD.C = 1e-2;
metaD.nBranch = 4;
metaD.nLevel = 1;
metaD.nBin = nBin;
metaD.useBias = 1;

metaD.initOption_pos = 'Global';
metaD.initOption_neg = 'Global';
metaD.featFiles_pos = posTrFiles;
metaD.featFiles_neg = negTrFiles;
PosTrFeats  = m_cmpInitFeats_2D(metaD, 'pos');
NegTrFeats  = m_cmpInitFeats_2D(metaD, 'neg');

metaD.featFiles_pos = posTstFiles;
metaD.featFiles_neg = negTstFiles;
PosTstFeats = m_cmpInitFeats_2D(metaD, 'pos');
NegTstFeats = m_cmpInitFeats_2D(metaD, 'neg');

%% Classification using global statistics
nPosTr = length(posTrFiles);
nNegTr = length(negTrFiles);
opts = sprintf('-t 0 -c %g -w1 %g -w-1 %g', metaD.C, 1/nPosTr, 1/nNegTr);
TrD = [PosTrFeats, NegTrFeats];
TrL = [ones(nPosTr,1); -ones(nNegTr,1)];
model = svmtrain(TrL, TrD', opts);
initW = full(model.SVs)'*model.sv_coef;
initB = -model.rho;

x = initW'*NegTrFeats + initB;
y = initW'*PosTrFeats + initB;
[rocArea, thres, acc] = ml_roc(x, y, 0);
fprintf('Results on train data, rocArea: %g, acc: %g\n', rocArea, acc);

x = initW'*NegTstFeats + initB;
y = initW'*PosTstFeats + initB;
[rocArea, thres, acc] = ml_roc(x, y, 0);
fprintf('Results on test data, rocArea: %g, acc: %g\n', rocArea, acc);


%% Joint localization and classification
metaD.initW = initW;
metaD.initOption_pos = 'initW';
metaD.runID = 'AnyRunID';
metaD.szRng_pos = [0; 50; 0; 30];
metaD.szRng_neg = [];


metaD.featFiles_pos = posTrFiles;
metaD.featFiles_neg = negTrFiles;
metaD.nNeg = length(metaD.featFiles_neg);
metaD.nPos = length(metaD.featFiles_pos);
metaD.cmpSegFeats  = 'm_cmpSegFeats_2D';
metaD.cmpInitFeats = 'm_cmpInitFeats_2D';
metaD.shldDisp_cmpSegFeat = 0;
metaD.b = 0;
startTime = clock;
[binW, b, objVals, wDeviation] = m_learnSegSVM(metaD); % learn the weight
fprintf('Total training time: %g\n', etime(clock, startTime));

%% Use the learned weights and display results
metaD.shldDisp_cmpSegFeat = 1;
metaD.b = b;

metaD.featFiles_pos = posTrFiles;
metaD.featFiles_neg = negTrFiles;
metaD.imFiles_pos = posTrFiles_im;
metaD.imFiles_neg = negTrFiles_im;
Feats = m_cmpSegFeats_2D(metaD, binW, 'pos');
y = binW(:)'*Feats + b;
Feats = m_cmpSegFeats_2D(metaD, binW, 'neg');
x = binW(:)'*Feats + b;
[rocArea, thres, acc] = ml_roc(x, y, 0);
fprintf('Results on train data, rocArea: %g, acc: %g\n', rocArea, acc);


metaD.featFiles_pos = posTstFiles;
metaD.featFiles_neg = negTstFiles;
metaD.imFiles_pos = posTstFiles_im;
metaD.imFiles_neg = negTstFiles_im;
Feats = m_cmpSegFeats_2D(metaD, binW, 'pos');
y = binW(:)'*Feats + b;
Feats = m_cmpSegFeats_2D(metaD, binW, 'neg');
x = binW(:)'*Feats + b;
[rocArea, thres, acc] = ml_roc(x, y, 0);
fprintf('Results on test data, rocArea: %g, acc: %g\n', rocArea, acc);


function [trFiles, tstFiles] = getTrTstLists(baseDir, classes, ...
    trIdxss, tstIdxss, fileExt)
    nClass = length(classes);

    trFiles = cell(0, 1);
    tstFiles = cell(0, 1);
    for i=1:nClass
        fileDir = [baseDir, classes{i}];
        files = ml_getFilesInDir(fileDir, fileExt);
        trFiles(end+1:end+length(trIdxss{i}),1) = files(trIdxss{i});
        tstFiles(end+1:end+length(tstIdxss{i}),1) = files(tstIdxss{i});
    end;
